#! /bin/sh

src_gcc="$HOME/source/gcc"
src_src="$HOME/source/cygnus.cygnus/src"
dst_root="$HOME/source/test"

dst_gcc="$dst_root/gcc"
dst_src="$dst_root/src"
dst_unified="$dst_root/unified"

setup=`echo $0 | sed -e 's,/[^/]*$,,'`

echo "Versions:"; echo

libtool_version=`libtool --version | grep GNU | sed -e 's/.*) //' -e 's/ .*//' 2>&1`
autoconf_version=`autoconf --version | grep GNU | sed -e 's/.*) //' -e 's/ .*//' 2>&1`
automake_version=`automake --version | grep GNU | sed -e 's/.*) //' -e 's/ .*//' 2>&1`
gettext_version=`gettext --version | grep GNU | sed -e 's/.*) //' -e 's/ .*//' 2>&1`
autogen_version=`autogen --version | grep autogen | sed -e 's/.*Ver\. //' 2>&1`

libtool_expected="1.4e"
autoconf_expected="2.57"
automake_expected="1.7.2"
gettext_expected="0.11.5"
autogen_expected="5.5"

ltmain_path=`type -p libtool | sed -e 's,libtool$,../share/libtool/ltmain.sh,'`

if [ "$libtool_version" != "$libtool_expected" ]; then
    echo "invalid libtool version: $libtool_version (was expecting $libtool_expected)"
    exit 1
fi
if [ "$autoconf_version" != "$autoconf_expected" ]; then
    echo "invalid autoconf version: $autoconf_version (was expecting $autoconf_expected)"
    exit 1
fi
if [ "$automake_version" != "$automake_expected" ]; then
    echo "invalid automake version: $automake_version (was expecting $automake_expected)"
    exit 1
fi
if [ "$gettext_version" != "$gettext_expected" ]; then
    echo "invalid gettext version: $gettext_version (was expecting $gettext_expected)"
    exit 1
fi
if [ "$autogen_version" != "$autogen_expected" ]; then
    echo "invalid autogen version: $autogen_version (was expecting $autogen_expected)"
    exit 1
fi

echo "libtool version: $libtool_version"
echo "autoconf version: $autoconf_version"
echo "automake version: $automake_version"
echo "gettext version: $gettext_version"
echo "autogen version: $autogen_version"
echo "ltmain.sh path: $ltmain_path"

echo; echo "Initializing:"; echo

mkdir -p "$dst_root"
(cd "$src_gcc" && rsync --progress -q -azvr --delete . "$dst_gcc")
(cd "$src_src" && rsync --progress -q -azvr --delete . "$dst_src")
rm -rf "$dst_unified"; mkdir -p "$dst_unified"

echo; echo "Patching:"; echo

for dir in "$dst_gcc" "$dst_src"; do

    cp -p /usr/share/automake-1.7/depcomp $dir
    cp -p /usr/share/automake-1.7/missing $dir
    cp -p /usr/share/automake-1.7/mkinstalldirs $dir
    cp -p /usr/share/automake-1.7/ylwrap $dir
    
    cp -p "$ltmain_path" "$dir"

    rm -f "$dir/ltcf-c.sh"
    rm -f "$dir/ltcf-cxx.sh"
    rm -f "$dir/ltcf-gcj.sh"
    rm -f "$dir/ltconfig"
    rm -f "$dir/libtool.m4"
done

rm -f "$dst_src/sim/ppc/aclocal.m4"
rm -f "$dst_gcc/boehm-gc/libtool.m4"

rm -f "$dst_gcc/libjava/libltdl/*"
cp -p /usr/share/libtool/libltdl/* "$dst_gcc/libjava/libltdl/" 

mv "$dst_gcc/libf2c/aclocal.m4" "$dst_gcc/libf2c/acinclude.m4"
mv "$dst_gcc/libobjc/aclocal.m4" "$dst_gcc/libobjc/acinclude.m4"
mv "$dst_gcc/libjava/gnu/gcj/runtime/natStringBuffer.cc" "$dst_gcc/libjava/gnu/gcj/runtime/natStringBufferg.cc"

# echo "Patching src-top:"
(cd "$dst_src" && patch -p0 < "$setup"/top-diffs.txt)
echo "Patching src:"
(cd "$dst_src" && patch -p0 < "$setup"/src-diffs.txt)
echo "Patching src-common:"
(cd "$dst_src" && patch -p0 < "$setup"/common-diffs.txt)

echo "Patching gcc-top:"
(cd "$dst_gcc" && patch -p0 < "$setup"/top-diffs.txt)
echo "Patching gcc:"
(cd "$dst_gcc" && patch -p0 < "$setup"/gcc-diffs.txt)
echo "Patching gcc-common:"
(cd "$dst_gcc" && patch -p0 < "$setup"/common-diffs.txt)

echo; echo "Merging:"; echo

configdirs_common="libiberty"
configdirs_src="bfd binutils gas gdb gprof ld mmalloc opcodes rda sim utils"
configdirs_gcc="boehm-gc fastjar gcc libf2c libffi libjava libobjc libstdc++-v3 zlib"

noconfigdirs_common="include texinfo config etc contrib"
noconfigdirs_src="blt cgen cpu dejagnu expect intl itcl iwidgets libgloss libgui newlib readline tcl tk winsup"
noconfigdirs_gcc="libbanshee libchill libio libmudflap libstdc++"

ignoredirs_common="contrib"
ignoredirs_src="sid"
ignoredirs_gcc=""

topfiles_common="ltmain.sh config.guess config.sub depcomp mkdep missing move-if-change ylwrap mkinstalldirs install-sh symlink-tree"
topfiles_common="$topfiles_common Makefile.def Makefile.in Makefile.tpl configure.in configure config-ml.in config.if"
topfiles_src="gettext.m4 "
topfiles_gcc=""

ignorefiles_common=""
ignorefiles_src=""
ignorefiles_gcc=""

for i in $configdirs_src $noconfigdirs_src $topfiles_src; do
    ln -s "$dst_src/$i" "$dst_unified"/
done

for i in $configdirs_gcc $noconfigdirs_gcc $topfiles_gcc; do
    ln -s "$dst_gcc/$i" "$dst_unified"/
done

for i in $configdirs_common $noconfigdirs_common $topfiles_common; do
    ln -s "$dst_gcc/$i" "$dst_unified"/
done

echo; echo "Configuring:"; echo

(cd $dst_src && autogen Makefile.def)
(cd $dst_gcc && autogen Makefile.def)

(cd $dst_src && autoconf)
(cd $dst_gcc && autoconf)

for i in $configdirs_src; do
    dirs=`find $dst_src/$i -type d`
    dirs=`for j in $dirs; do if [ -f $j/configure.in ]; then echo $j; fi; done`
    for j in $dirs; do
        echo "Configuring $j"
	autoreconf -f -Wno-obsolete -I"$dst_src" $j
    done
done

for i in $configdirs_gcc; do
    dirs=`find $dst_gcc/$i -type d`
    dirs=`for j in $dirs; do if [ -f $j/configure.in ]; then echo $j; fi; done`
    for j in $dirs; do
        echo "Configuring $j"
	autoreconf -f -Wno-obsolete -I"$dst_gcc" $j
    done
done
